@@ -65,7 +65,14 @@ sub Commit {    # XXX: from custom prepare code
     my $self = shift;
     if ( my ($rejected) =
         $self->TicketObj->AllDependedOnBy( Type => 'ticket' ) ) {
-        my $note = $self->GetNotes;
+        my $note = '';
+        if ( RT->Config->Get('ApprovalRejectionNotes') ) {
+            my $t = $self->TicketObj->Transactions;
+            while ( my $o = $t->Next ) {
+                next unless $o->Type eq 'Correspond';
+                $note .= $o->Content . "\n" if $o->ContentObj;
+            }
+        }
 
         my $template = $self->GetTemplate('Approval Rejected',
                                           TicketObj => $rejected,
@@ -75,7 +82,7 @@ sub Commit {    # XXX: from custom prepare code
 
         $rejected->Correspond( MIMEObj => $template->MIMEObj );
         $rejected->SetStatus(
-            Status => 'rejected',
+            Status => $rejected->QueueObj->Lifecycle->DefaultStatus('denied') || 'rejected',
             Force  => 1,
         );
     }
@@ -105,20 +112,4 @@ sub Commit {    # XXX: from custom prepare code
 
 }
 
-sub GetNotes {
-    my $self = shift;
-    my $note = '';
-
-    if ( RT->Config->Get('ApprovalRejectionNotes') ) {
-        my $t = $self->TicketObj->Transactions;
-        while ( my $o = $t->Next ) {
-            next unless $o->Type eq 'Correspond';
-            $note .= $o->Content . "\n" if $o->ContentObj;
-        }
-    }
-    return $note;
-}
-
-RT::Base->_ImportOverlays();
-
 1;
