@@ -137,7 +137,7 @@ sub __BuildLimitCheck {
             return 0 unless $value ne $condition;
         } elsif (uc($op) eq "LIKE") {
             return 0 unless $value =~ /\Q$condition\E/i;
-        } elsif (rc($op) eq "NOT LIKE") {
+        } elsif (uc($op) eq "NOT LIKE") {
             return 0 unless $value !~ /\Q$condition\E/i;
         } else {
             return 0;
@@ -179,9 +179,9 @@ sub _DoSearch {
             customfield => $self->{'__external_cf'},
             sortorder => 0,
             description => '',
-            creator => $RT::SystemUser->id,
+            creator => RT->SystemUser->id,
             created => undef,
-            lastupdatedby => $RT::SystemUser->id,
+            lastupdatedby => RT->SystemUser->id,
             lastupdated => undef,
     );
 
