@@ -51,6 +51,7 @@ package RT::Test;
 use strict;
 use warnings;
 
+
 use base 'Test::More';
 
 use Socket;
@@ -58,24 +59,7 @@ use File::Temp qw(tempfile);
 use File::Path qw(mkpath);
 use File::Spec;
 
-our $SKIP_REQUEST_WORK_AROUND = 0;
-
-use HTTP::Request::Common ();
-use Hook::LexWrap;
-wrap 'HTTP::Request::Common::form_data',
-   post => sub {
-       return if $SKIP_REQUEST_WORK_AROUND;
-       my $data = $_[-1];
-       if (ref $data) {
-       $data->[0] = Encode::encode_utf8($data->[0]);
-       }
-       else {
-       $_[-1] = Encode::encode_utf8($_[-1]);
-       }
-   };
-
-
-our @EXPORT = qw(is_empty parse_mail);
+our @EXPORT = qw(is_empty diag parse_mail works fails);
 our ($port, $dbname);
 our @SERVERS;
 
@@ -88,6 +72,8 @@ my %tmp = (
     mailbox   => undef,
 );
 
+my %rttest_opt;
+
 =head1 NAME
 
 RT::Test - RT Testing
@@ -125,24 +111,34 @@ sub generate_port {
 
 BEGIN {
     $port   = generate_port();
-    $dbname = $ENV{RT_TEST_PARALLEL}? "rt3test_$port" : "rt3test";
+    $dbname = $ENV{RT_TEST_PARALLEL}? "rt4test_$port" : "rt4test";
 };
 
 sub import {
     my $class = shift;
-    my %args = @_;
+    my %args = %rttest_opt = @_;
 
     # Spit out a plan (if we got one) *before* we load modules
     if ( $args{'tests'} ) {
         $class->builder->plan( tests => $args{'tests'} )
           unless $args{'tests'} eq 'no_declare';
     }
+    elsif ( exists $args{'tests'} ) {
+        # do nothing if they say "tests => undef" - let them make the plan
+    }
     else {
         $class->builder->no_plan unless $class->builder->has_plan;
     }
 
+    push @{ $args{'plugins'} ||= [] }, @{ $args{'requires'} }
+        if $args{'requires'};
+    push @{ $args{'plugins'} ||= [] }, $args{'testing'}
+        if $args{'testing'};
+
     $class->bootstrap_tempdir;
 
+    $class->bootstrap_plugins_paths( %args );
+
     $class->bootstrap_config( %args );
 
     use RT;
@@ -152,9 +148,18 @@ sub import {
 
     $class->bootstrap_db( %args );
 
-    RT->Init;
+    RT::InitPluginPaths();
 
-    $class->bootstrap_plugins( %args );
+    __reconnect_rt()
+        unless $args{nodb};
+
+    RT::InitClasses();
+    RT::InitLogging();
+
+    RT->Plugins;
+
+    RT::I18N->Init();
+    RT->Config->PostLoadCheck;
 
     $class->set_config_wrapper;
 
@@ -178,6 +183,12 @@ sub import {
     }
 
     Test::More->export_to_level($level);
+
+    # blow away their diag so we can redefine it without warning
+    # better than "no warnings 'redefine'" because we might accidentally
+    # suppress a mistaken redefinition
+    no strict 'refs';
+    delete ${ caller($level) . '::' }{diag};
     __PACKAGE__->export_to_level($level);
 }
 
@@ -224,7 +235,8 @@ sub bootstrap_config {
 Set( \$WebDomain, "localhost");
 Set( \$WebPort,   $port);
 Set( \$WebPath,   "");
-Set( \$RTAddressRegexp , qr/^bad_re_that_doesnt_match\$/);
+Set( \@LexiconLanguages, qw(en zh_TW fr ja));
+Set( \$RTAddressRegexp , qr/^bad_re_that_doesnt_match\$/i);
 };
     if ( $ENV{'RT_TEST_DB_SID'} ) { # oracle case
         print $config "Set( \$DatabaseName , '$ENV{'RT_TEST_DB_SID'}' );\n";
@@ -233,8 +245,20 @@ Set( \$RTAddressRegexp , qr/^bad_re_that_doesnt_match\$/);
         print $config "Set( \$DatabaseName , '$dbname');\n";
         print $config "Set( \$DatabaseUser , 'u${dbname}');\n";
     }
-    print $config "Set( \$DevelMode, 0 );\n"
-        if $INC{'Devel/Cover.pm'};
+
+    if ( $args{'plugins'} ) {
+        print $config "Set( \@Plugins, qw(". join( ' ', @{ $args{'plugins'} } ) .") );\n";
+    }
+
+    if ( $INC{'Devel/Cover.pm'} ) {
+        print $config "Set( \$DevelMode, 0 );\n";
+    }
+    elsif ( $ENV{RT_TEST_DEVEL} ) {
+        print $config "Set( \$DevelMode, 1 );\n";
+    }
+    else {
+        print $config "Set( \$DevelMode, 0 );\n";
+    }
 
     $self->bootstrap_logging( $config );
 
@@ -254,7 +278,9 @@ Set( \$MailCommand, sub {
     close \$handle;
 } );
 END
-    
+
+    $self->bootstrap_more_config($config, \%args);
+
     print $config $args{'config'} if $args{'config'};
 
     print $config "\n1;\n";
@@ -264,6 +290,8 @@ END
     return $config;
 }
 
+sub bootstrap_more_config { }
+
 sub bootstrap_logging {
     my $self = shift;
     my $config = shift;
@@ -293,8 +321,15 @@ sub set_config_wrapper {
     my $old_sub = \&RT::Config::Set;
     no warnings 'redefine';
     *RT::Config::Set = sub {
-        my @caller = caller;
-        if ( ($caller[1]||'') =~ /\.t$/ ) {
+        # Determine if the caller is either from a test script, or
+        # from helper functions called by test script to alter
+        # configuration that should be written.  This is necessary
+        # because some extensions (RTIR, for example) temporarily swap
+        # configuration values out and back in Mason during requests.
+        my @caller = caller(1); # preserve list context
+        @caller = caller(0) unless @caller;
+
+        if ( ($caller[1]||'') =~ /\.t$/) {
             my ($self, $name) = @_;
             my $type = $RT::Config::META{$name}->{'Type'} || 'SCALAR';
             my %sigils = (
@@ -306,10 +341,11 @@ sub set_config_wrapper {
             open( my $fh, '>>', $tmp{'config'}{'RT'} )
                 or die "Couldn't open config file: $!";
             require Data::Dumper;
+            local $Data::Dumper::Terse = 1;
             my $dump = Data::Dumper::Dumper([@_[2 .. $#_]]);
             $dump =~ s/;\s+$//;
             print $fh
-                "\nSet(${sigil}${name}, \@{". $dump ."}); 1;\n";
+                "\nSet(${sigil}${name}, \@{". $dump ."});\n1;\n";
             close $fh;
 
             if ( @SERVERS ) {
@@ -333,67 +369,45 @@ sub bootstrap_db {
     }
 
     require RT::Handle;
-    # bootstrap with dba cred
-    my $dbh = _get_dbh(RT::Handle->SystemDSN,
-               $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD});
-
-    unless ( $ENV{RT_TEST_PARALLEL} ) {
-        # already dropped db in parallel tests, need to do so for other cases.
-        RT::Handle->DropDatabase( $dbh, Force => 1 );
+    if (my $forceopt = $ENV{RT_TEST_FORCE_OPT}) {
+        Test::More::diag "forcing $forceopt";
+        $args{$forceopt}=1;
     }
 
-    RT::Handle->CreateDatabase( $dbh );
-    $dbh->disconnect;
-    $created_new_db++;
-
-    $dbh = _get_dbh(RT::Handle->DSN,
-            $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD});
-
-    $RT::Handle = new RT::Handle;
-    $RT::Handle->dbh( $dbh );
-    $RT::Handle->InsertSchema( $dbh );
+    return if $args{nodb};
 
     my $db_type = RT->Config->Get('DatabaseType');
-    $RT::Handle->InsertACL( $dbh ) unless $db_type eq 'Oracle';
+    __create_database();
+    __reconnect_rt('as dba');
+    $RT::Handle->InsertSchema;
+    $RT::Handle->InsertACL unless $db_type eq 'Oracle';
 
-    $RT::Handle = new RT::Handle;
-    $RT::Handle->dbh( undef );
-    RT->ConnectToDatabase;
     RT->InitLogging;
-    RT->InitSystemObjects;
-    $RT::Handle->InsertInitialData;
+    __reconnect_rt();
 
-    DBIx::SearchBuilder::Record::Cachable->FlushCache;
-    $RT::Handle = new RT::Handle;
-    $RT::Handle->dbh( undef );
-    RT->Init;
+    $RT::Handle->InsertInitialData
+        unless $args{noinitialdata};
 
-    $RT::Handle->PrintError;
-    $RT::Handle->dbh->{PrintError} = 1;
+    $RT::Handle->InsertData( $RT::EtcPath . "/initialdata" )
+        unless $args{noinitialdata} or $args{nodata};
 
-    unless ( $args{'nodata'} ) {
-        $RT::Handle->InsertData( $RT::EtcPath . "/initialdata" );
-    }
-    DBIx::SearchBuilder::Record::Cachable->FlushCache;
+    $self->bootstrap_plugins_db( %args );
 }
 
-sub bootstrap_plugins {
+sub bootstrap_plugins_paths {
     my $self = shift;
     my %args = @_;
 
-    return unless $args{'requires'};
+    return unless $args{'plugins'};
+    my @plugins = @{ $args{'plugins'} };
 
-    my @plugins = @{ $args{'requires'} };
-    push @plugins, $args{'testing'}
-        if $args{'testing'};
-
-    require RT::Plugin;
     my $cwd;
     if ( $args{'testing'} ) {
         require Cwd;
         $cwd = Cwd::getcwd();
     }
 
+    require RT::Plugin;
     my $old_func = \&RT::Plugin::_BasePath;
     no warnings 'redefine';
     *RT::Plugin::_BasePath = sub {
@@ -408,17 +422,17 @@ sub bootstrap_plugins {
         }
         return $old_func->(@_);
     };
+}
 
-    RT->Config->Set( Plugins => @plugins );
-    RT->InitPluginPaths;
+sub bootstrap_plugins_db {
+    my $self = shift;
+    my %args = @_;
 
-    my $dba_dbh;
-    $dba_dbh = _get_dbh(
-        RT::Handle->DSN,
-        $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD},
-    ) if @plugins;
+    return unless $args{'plugins'};
 
     require File::Spec;
+
+    my @plugins = @{ $args{'plugins'} };
     foreach my $name ( @plugins ) {
         my $plugin = RT::Plugin->new( name => $name );
         Test::More::diag( "Initializing DB for the $name plugin" )
@@ -428,31 +442,37 @@ sub bootstrap_plugins {
         Test::More::diag( "etc path of the plugin is '$etc_path'" )
             if $ENV{'TEST_VERBOSE'};
 
-        if ( -e $etc_path ) {
-            my ($ret, $msg) = $RT::Handle->InsertSchema( $dba_dbh, $etc_path );
-            Test::More::ok($ret || $msg =~ /^Couldn't find schema/, "Created schema: ".($msg||''));
-
-            ($ret, $msg) = $RT::Handle->InsertACL( $dba_dbh, $etc_path );
-            Test::More::ok($ret || $msg =~ /^Couldn't find ACLs/, "Created ACL: ".($msg||''));
-
-            my $data_file = File::Spec->catfile( $etc_path, 'initialdata' );
-            if ( -e $data_file ) {
-                ($ret, $msg) = $RT::Handle->InsertData( $data_file );;
-                Test::More::ok($ret, "Inserted data".($msg||''));
-            } else {
-                Test::More::ok(1, "There is no data file" );
-            }
-        }
-        else {
-# we can not say if plugin has no data or we screwed with etc path
+        unless ( -e $etc_path ) {
+            # We can't tell if the plugin has no data, or we screwed up the etc/ path
             Test::More::ok(1, "There is no etc dir: no schema" );
             Test::More::ok(1, "There is no etc dir: no ACLs" );
             Test::More::ok(1, "There is no etc dir: no data" );
+            next;
         }
 
-        $RT::Handle->Connect; # XXX: strange but mysql can loose connection
+        __reconnect_rt('as dba');
+
+        { # schema
+            my ($ret, $msg) = $RT::Handle->InsertSchema( undef, $etc_path );
+            Test::More::ok($ret || $msg =~ /^Couldn't find schema/, "Created schema: ".($msg||''));
+        }
+
+        { # ACLs
+            my ($ret, $msg) = $RT::Handle->InsertACL( undef, $etc_path );
+            Test::More::ok($ret || $msg =~ /^Couldn't find ACLs/, "Created ACL: ".($msg||''));
+        }
+
+        # data
+        my $data_file = File::Spec->catfile( $etc_path, 'initialdata' );
+        if ( -e $data_file ) {
+            __reconnect_rt();
+            my ($ret, $msg) = $RT::Handle->InsertData( $data_file );;
+            Test::More::ok($ret, "Inserted data".($msg||''));
+        } else {
+            Test::More::ok(1, "There is no data file" );
+        }
     }
-    $dba_dbh->disconnect if $dba_dbh;
+    __reconnect_rt();
 }
 
 sub _get_dbh {
@@ -472,6 +492,72 @@ sub _get_dbh {
     return $dbh;
 }
 
+sub __create_database {
+    # bootstrap with dba cred
+    my $dbh = _get_dbh(
+        RT::Handle->SystemDSN,
+        $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD}
+    );
+
+    unless ( $ENV{RT_TEST_PARALLEL} ) {
+        # already dropped db in parallel tests, need to do so for other cases.
+        __drop_database( $dbh );
+
+    }
+    RT::Handle->CreateDatabase( $dbh );
+    $dbh->disconnect;
+    $created_new_db++;
+}
+
+sub __drop_database {
+    my $dbh = shift;
+
+    # Pg doesn't like if you issue a DROP DATABASE while still connected
+    # it's still may fail if web-server is out there and holding a connection
+    __disconnect_rt();
+
+    my $my_dbh = $dbh? 0 : 1;
+    $dbh ||= _get_dbh(
+        RT::Handle->SystemDSN,
+        $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD}
+    );
+    RT::Handle->DropDatabase( $dbh );
+    $dbh->disconnect if $my_dbh;
+}
+
+sub __reconnect_rt {
+    my $as_dba = shift;
+    __disconnect_rt();
+
+    # look at %DBIHandle and $PrevHandle in DBIx::SB::Handle for explanation
+    $RT::Handle = RT::Handle->new;
+    $RT::Handle->dbh( undef );
+    $RT::Handle->Connect(
+        $as_dba
+        ? (User => $ENV{RT_DBA_USER}, Password => $ENV{RT_DBA_PASSWORD})
+        : ()
+    );
+    $RT::Handle->PrintError;
+    $RT::Handle->dbh->{PrintError} = 1;
+    return $RT::Handle->dbh;
+}
+
+sub __disconnect_rt {
+    # look at %DBIHandle and $PrevHandle in DBIx::SB::Handle for explanation
+    $RT::Handle->dbh->disconnect if $RT::Handle and $RT::Handle->dbh;
+
+    %DBIx::SearchBuilder::Handle::DBIHandle = ();
+    $DBIx::SearchBuilder::Handle::PrevHandle = undef;
+
+    $RT::Handle = undef;
+
+    delete $RT::System->{attributes};
+
+    DBIx::SearchBuilder::Record::Cachable->FlushCache
+          if DBIx::SearchBuilder::Record::Cachable->can("FlushCache");
+}
+
+
 =head1 UTILITIES
 
 =head2 load_or_create_user
@@ -486,7 +572,7 @@ sub load_or_create_user {
     $MemberOf = [ $MemberOf ] if defined $MemberOf && !ref $MemberOf;
     $MemberOf ||= [];
 
-    my $obj = RT::User->new( $RT::SystemUser );
+    my $obj = RT::User->new( RT->SystemUser );
     if ( $args{'Name'} ) {
         $obj->LoadByCols( Name => $args{'Name'} );
     } elsif ( $args{'EmailAddress'} ) {
@@ -508,7 +594,7 @@ sub load_or_create_user {
     # clean group membership
     {
         require RT::GroupMembers;
-        my $gms = RT::GroupMembers->new( $RT::SystemUser );
+        my $gms = RT::GroupMembers->new( RT->SystemUser );
         my $groups_alias = $gms->Join(
             FIELD1 => 'GroupId', TABLE2 => 'Groups', FIELD2 => 'id',
         );
@@ -537,7 +623,7 @@ sub load_or_create_user {
 sub load_or_create_queue {
     my $self = shift;
     my %args = ( Disabled => 0, @_ );
-    my $obj = RT::Queue->new( $RT::SystemUser );
+    my $obj = RT::Queue->new( RT->SystemUser );
     if ( $args{'Name'} ) {
         $obj->LoadByCols( Name => $args{'Name'} );
     } else {
@@ -550,7 +636,7 @@ sub load_or_create_queue {
         my @fields = qw(CorrespondAddress CommentAddress);
         foreach my $field ( @fields ) {
             next unless exists $args{ $field };
-            next if $args{ $field } eq $obj->$field;
+            next if $args{ $field } eq ($obj->$field || '');
             
             no warnings 'uninitialized';
             my $method = 'Set'. $field;
@@ -562,6 +648,111 @@ sub load_or_create_queue {
     return $obj;
 }
 
+sub delete_queue_watchers {
+    my $self = shift;
+    my @queues = @_;
+
+    foreach my $q ( @queues ) {
+        foreach my $t (qw(Cc AdminCc) ) {
+            $q->DeleteWatcher( Type => $t, PrincipalId => $_->MemberId )
+                foreach @{ $q->$t()->MembersObj->ItemsArrayRef };
+        }
+    }
+}
+
+sub create_tickets {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my $defaults = shift;
+    my @data = @_;
+    @data = sort { rand(100) <=> rand(100) } @data
+        if delete $defaults->{'RandomOrder'};
+
+    $defaults->{'Queue'} ||= 'General';
+
+    my @res = ();
+    while ( @data ) {
+        my %args = %{ shift @data };
+        $args{$_} = $res[ $args{$_} ]->id foreach
+            grep $args{ $_ }, keys %RT::Ticket::LINKTYPEMAP;
+        push @res, $self->create_ticket( %$defaults, %args );
+    }
+    return @res;
+}
+
+sub create_ticket {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my %args = @_;
+
+    if ($args{Queue} && $args{Queue} =~ /\D/) {
+        my $queue = RT::Queue->new(RT->SystemUser);
+        if (my $id = $queue->Load($args{Queue}) ) {
+            $args{Queue} = $id;
+        } else {
+            die ("Error: Invalid queue $args{Queue}");
+        }
+    }
+
+    if ( my $content = delete $args{'Content'} ) {
+        $args{'MIMEObj'} = MIME::Entity->build(
+            From    => $args{'Requestor'},
+            Subject => $args{'Subject'},
+            Data    => $content,
+        );
+    }
+
+    my $ticket = RT::Ticket->new( RT->SystemUser );
+    my ( $id, undef, $msg ) = $ticket->Create( %args );
+    Test::More::ok( $id, "ticket created" )
+        or Test::More::diag("error: $msg");
+
+    # hackish, but simpler
+    if ( $args{'LastUpdatedBy'} ) {
+        $ticket->__Set( Field => 'LastUpdatedBy', Value => $args{'LastUpdatedBy'} );
+    }
+
+
+    for my $field ( keys %args ) {
+        #TODO check links and watchers
+
+        if ( $field =~ /CustomField-(\d+)/ ) {
+            my $cf = $1;
+            my $got = join ',', sort map $_->Content,
+                @{ $ticket->CustomFieldValues($cf)->ItemsArrayRef };
+            my $expected = ref $args{$field}
+                ? join( ',', sort @{ $args{$field} } )
+                : $args{$field};
+            Test::More::is( $got, $expected, 'correct CF values' );
+        }
+        else {
+            next if ref $args{$field};
+            next unless $ticket->can($field) or $ticket->_Accessible($field,"read");
+            next if ref $ticket->$field();
+            Test::More::is( $ticket->$field(), $args{$field}, "$field is correct" );
+        }
+    }
+
+    return $ticket;
+}
+
+sub delete_tickets {
+    my $self = shift;
+    my $query = shift;
+    my $tickets = RT::Tickets->new( RT->SystemUser );
+    if ( $query ) {
+        $tickets->FromSQL( $query );
+    }
+    else {
+        $tickets->UnLimit;
+    }
+    while ( my $ticket = $tickets->Next ) {
+        $ticket->Delete;
+    }
+}
+
 =head2 load_or_create_custom_field
 
 =cut
@@ -569,7 +760,7 @@ sub load_or_create_queue {
 sub load_or_create_custom_field {
     my $self = shift;
     my %args = ( Disabled => 0, @_ );
-    my $obj = RT::CustomField->new( $RT::SystemUser );
+    my $obj = RT::CustomField->new( RT->SystemUser );
     if ( $args{'Name'} ) {
         $obj->LoadByName( Name => $args{'Name'}, Queue => $args{'Queue'} );
     } else {
@@ -586,7 +777,7 @@ sub load_or_create_custom_field {
 sub last_ticket {
     my $self = shift;
     my $current = shift;
-    $current = $current ? RT::CurrentUser->new($current) : $RT::SystemUser;
+    $current = $current ? RT::CurrentUser->new($current) : RT->SystemUser;
     my $tickets = RT::Tickets->new( $current );
     $tickets->OrderBy( FIELD => 'id', ORDER => 'DESC' );
     $tickets->Limit( FIELD => 'id', OPERATOR => '>', VALUE => '0' );
@@ -599,17 +790,17 @@ sub store_rights {
 
     require RT::ACE;
     # fake construction
-    RT::ACE->new( $RT::SystemUser );
+    RT::ACE->new( RT->SystemUser );
     my @fields = keys %{ RT::ACE->_ClassAccessible };
 
     require RT::ACL;
-    my $acl = RT::ACL->new( $RT::SystemUser );
+    my $acl = RT::ACL->new( RT->SystemUser );
     $acl->Limit( FIELD => 'RightName', OPERATOR => '!=', VALUE => 'SuperUser' );
 
     my @res;
     while ( my $ace = $acl->Next ) {
         my $obj = $ace->PrincipalObj->Object;
-        if ( $obj->isa('RT::Group') && $obj->Type eq 'UserEquiv' && $obj->Instance == $RT::Nobody->id ) {
+        if ( $obj->isa('RT::Group') && $obj->Type eq 'UserEquiv' && $obj->Instance == RT->Nobody->id ) {
             next;
         }
 
@@ -626,7 +817,7 @@ sub restore_rights {
     my $self = shift;
     my @entries = @_;
     foreach my $entry ( @entries ) {
-        my $ace = RT::ACE->new( $RT::SystemUser );
+        my $ace = RT::ACE->new( RT->SystemUser );
         my ($status, $msg) = $ace->RT::Record::Create( %$entry );
         unless ( $status ) {
             Test::More::diag "couldn't create a record: $msg";
@@ -638,11 +829,11 @@ sub set_rights {
     my $self = shift;
 
     require RT::ACL;
-    my $acl = RT::ACL->new( $RT::SystemUser );
+    my $acl = RT::ACL->new( RT->SystemUser );
     $acl->Limit( FIELD => 'RightName', OPERATOR => '!=', VALUE => 'SuperUser' );
     while ( my $ace = $acl->Next ) {
         my $obj = $ace->PrincipalObj->Object;
-        if ( $obj->isa('RT::Group') && $obj->Type eq 'UserEquiv' && $obj->Instance == $RT::Nobody->id ) {
+        if ( $obj->isa('RT::Group') && $obj->Type eq 'UserEquiv' && $obj->Instance == RT->Nobody->id ) {
             next;
         }
         $ace->Delete;
@@ -659,10 +850,10 @@ sub add_rights {
         my $principal = delete $e->{'Principal'};
         unless ( ref $principal ) {
             if ( $principal =~ /^(everyone|(?:un)?privileged)$/i ) {
-                $principal = RT::Group->new( $RT::SystemUser );
+                $principal = RT::Group->new( RT->SystemUser );
                 $principal->LoadSystemInternalGroup($1);
             } elsif ( $principal =~ /^(Owner|Requestor|(?:Admin)?Cc)$/i ) {
-                $principal = RT::Group->new( $RT::SystemUser );
+                $principal = RT::Group->new( RT->SystemUser );
                 $principal->LoadByCols(
                     Domain => (ref($e->{'Object'})||'RT::System').'-Role',
                     Type => $1,
@@ -746,7 +937,7 @@ sub run_and_capture {
     return ($?, $result);
 }
 
-sub send_via_mailgate {
+sub send_via_mailgate_and_http {
     my $self = shift;
     my $message = shift;
     my %args = (@_);
@@ -769,6 +960,26 @@ sub send_via_mailgate {
     return ($status, $id);
 }
 
+
+sub send_via_mailgate {
+    my $self    = shift;
+    my $message = shift;
+    my %args = ( action => 'correspond',
+                 queue  => 'General',
+                 @_
+               );
+
+    if ( UNIVERSAL::isa( $message, 'MIME::Entity' ) ) {
+        $message = $message->as_string;
+    }
+
+    my ( $status, $error_message, $ticket )
+        = RT::Interface::Email::Gateway( {%args, message => $message} );
+    return ( $status, $ticket->id );
+
+}
+
+
 sub open_mailgate_ok {
     my $class   = shift;
     my $baseurl = shift;
@@ -803,11 +1014,6 @@ sub mailsent_ok {
     );
 }
 
-sub set_mail_catcher {
-    my $self = shift;
-    return 1;
-}
-
 sub fetch_caught_mails {
     my $self = shift;
     return grep /\S/, split /%% split me! %%\n/,
@@ -864,6 +1070,14 @@ sub get_abs_relocatable_dir {
     }
 }
 
+sub gnupg_homedir {
+    my $self = shift;
+    File::Temp->newdir(
+        DIR => $tmp{directory},
+        CLEANUP => 0,
+    );
+}
+
 sub import_gnupg_key {
     my $self = shift;
     my $key  = shift;
@@ -902,7 +1116,7 @@ sub lsign_gnupg_key {
     my $key = shift;
 
     require RT::Crypt::GnuPG; require GnuPG::Interface;
-    my $gnupg = new GnuPG::Interface;
+    my $gnupg = GnuPG::Interface->new();
     my %opt = RT->Config->Get('GnuPGOptions');
     $gnupg->options->hash_init(
         RT::Crypt::GnuPG::_PrepareGnuPGOptions( %opt ),
@@ -911,12 +1125,12 @@ sub lsign_gnupg_key {
 
     my %handle; 
     my $handles = GnuPG::Handles->new(
-        stdin   => ($handle{'input'}   = new IO::Handle),
-        stdout  => ($handle{'output'}  = new IO::Handle),
-        stderr  => ($handle{'error'}   = new IO::Handle),
-        logger  => ($handle{'logger'}  = new IO::Handle),
-        status  => ($handle{'status'}  = new IO::Handle),
-        command => ($handle{'command'} = new IO::Handle),
+        stdin   => ($handle{'input'}   = IO::Handle->new()),
+        stdout  => ($handle{'output'}  = IO::Handle->new()),
+        stderr  => ($handle{'error'}   = IO::Handle->new()),
+        logger  => ($handle{'logger'}  = IO::Handle->new()),
+        status  => ($handle{'status'}  = IO::Handle->new()),
+        command => ($handle{'command'} = IO::Handle->new()),
     );
 
     eval {
@@ -959,7 +1173,7 @@ sub trust_gnupg_key {
     my $key = shift;
 
     require RT::Crypt::GnuPG; require GnuPG::Interface;
-    my $gnupg = new GnuPG::Interface;
+    my $gnupg = GnuPG::Interface->new();
     my %opt = RT->Config->Get('GnuPGOptions');
     $gnupg->options->hash_init(
         RT::Crypt::GnuPG::_PrepareGnuPGOptions( %opt ),
@@ -968,12 +1182,12 @@ sub trust_gnupg_key {
 
     my %handle; 
     my $handles = GnuPG::Handles->new(
-        stdin   => ($handle{'input'}   = new IO::Handle),
-        stdout  => ($handle{'output'}  = new IO::Handle),
-        stderr  => ($handle{'error'}   = new IO::Handle),
-        logger  => ($handle{'logger'}  = new IO::Handle),
-        status  => ($handle{'status'}  = new IO::Handle),
-        command => ($handle{'command'} = new IO::Handle),
+        stdin   => ($handle{'input'}   = IO::Handle->new()),
+        stdout  => ($handle{'output'}  = IO::Handle->new()),
+        stderr  => ($handle{'error'}   = IO::Handle->new()),
+        logger  => ($handle{'logger'}  = IO::Handle->new()),
+        status  => ($handle{'status'}  = IO::Handle->new()),
+        command => ($handle{'command'} = IO::Handle->new()),
     );
 
     eval {
@@ -1027,7 +1241,15 @@ sub started_ok {
 
     require RT::Test::Web;
 
-    my $which = $ENV{'RT_TEST_WEB_HANDLER'} || 'standalone';
+    if ($rttest_opt{nodb}) {
+        die "you are trying to use a test web server without db, try use noinitialdata => 1 instead";
+    }
+
+
+    $ENV{'RT_TEST_WEB_HANDLER'} = undef
+        if $rttest_opt{actual_server} && ($ENV{'RT_TEST_WEB_HANDLER'}||'') eq 'inline';
+    $ENV{'RT_TEST_WEB_HANDLER'} ||= 'plack';
+    my $which = $ENV{'RT_TEST_WEB_HANDLER'};
     my ($server, $variant) = split /\+/, $which, 2;
 
     my $function = 'start_'. $server .'_server';
@@ -1037,185 +1259,109 @@ sub started_ok {
     return $self->$function( $variant, @_ );
 }
 
-sub start_standalone_server {
+sub start_plack_server {
     my $self = shift;
 
+    require Plack::Loader;
+    my $plack_server = Plack::Loader->load
+        ('Standalone',
+         port => $port,
+         server_ready => sub {
+             kill 'USR1' => getppid();
+         });
 
-    require RT::Interface::Web::Standalone;
+    # We are expecting a USR1 from the child process after it's ready
+    # to listen.  We set this up _before_ we fork to avoid race
+    # conditions.
+    my $handled;
+    local $SIG{USR1} = sub { $handled = 1};
 
-    require Test::HTTP::Server::Simple::StashWarnings;
-    unshift @RT::Interface::Web::Standalone::ISA,
-        'Test::HTTP::Server::Simple::StashWarnings';
-    *RT::Interface::Web::Standalone::test_warning_path = sub {
-        "/__test_warnings";
+    __disconnect_rt();
+    my $pid = fork();
+    die "failed to fork" unless defined $pid;
+
+    if ($pid) {
+        sleep 15 unless $handled;
+        Test::More::diag "did not get expected USR1 for test server readiness"
+            unless $handled;
+        push @SERVERS, $pid;
+        my $Tester = Test::Builder->new;
+        $Tester->ok(1, @_);
+
+        __reconnect_rt();
+        return ("http://localhost:$port", RT::Test::Web->new);
+    }
+
+    require POSIX;
+    if ( $^O !~ /MSWin32/ ) {
+        POSIX::setsid()
+            or die "Can't start a new session: $!";
+    }
+
+    # stick this in a scope so that when $app is garbage collected,
+    # StashWarnings can complain about unhandled warnings
+    do {
+        require RT::Interface::Web::Handler;
+        my $app = RT::Interface::Web::Handler->PSGIApp;
+
+        require Plack::Middleware::Test::StashWarnings;
+        $app = Plack::Middleware::Test::StashWarnings->wrap($app);
+
+        $plack_server->run($app);
     };
 
-    my $s = RT::Interface::Web::Standalone->new($port);
+    exit;
+}
 
-    my $ret = $s->started_ok;
-    push @SERVERS, $s->pids;
+sub start_inline_server {
+    my $self = shift;
 
-    $RT::Handle = new RT::Handle;
-    $RT::Handle->dbh( undef );
-    RT->ConnectToDatabase;
+    require Test::WWW::Mechanize::PSGI;
+    unshift @RT::Test::Web::ISA, 'Test::WWW::Mechanize::PSGI';
 
-    return ($ret, RT::Test::Web->new);
+    # Clear out squished CSS and JS cache, since it's retained across
+    # servers, since it's in-process
+    RT::Interface::Web->ClearSquished;
+
+    Test::More::ok(1, "psgi test server ok");
+    return ("http://localhost:$port", RT::Test::Web->new);
 }
 
 sub start_apache_server {
     my $self = shift;
     my $variant = shift || 'mod_perl';
+    $ENV{RT_TEST_WEB_HANDLER} = "apache+$variant";
 
-    my %info = $self->apache_server_info( variant => $variant );
-
-    Test::More::diag(do {
-        open( my $fh, '<', $tmp{'config'}{'RT'} ) or die $!;
-        local $/;
-        <$fh>
-    });
-
-    my $tmpl = File::Spec->rel2abs( File::Spec->catfile(
-        't', 'data', 'configs',
-        'apache'. $info{'version'} .'+'. $variant .'.conf'
-    ) );
-    my %opt = (
-        listen         => $port,
-        server_root    => $info{'HTTPD_ROOT'} || $ENV{'HTTPD_ROOT'}
-            || Test::More::BAIL_OUT("Couldn't figure out server root"),
-        document_root  => $RT::MasonComponentRoot,
-        tmp_dir        => "$tmp{'directory'}",
-        rt_bin_path    => $RT::BinPath,
-        rt_site_config => $ENV{'RT_SITE_CONFIG'},
-    );
-    foreach (qw(log pid lock)) {
-        $opt{$_ .'_file'} = File::Spec->catfile(
-            "$tmp{'directory'}", "apache.$_"
-        );
-    }
-    {
-        my $method = 'apache_'.$variant.'_server_options';
-        $self->$method( \%info, \%opt );
-    }
-    $tmp{'config'}{'apache'} = File::Spec->catfile(
-        "$tmp{'directory'}", "apache.conf"
-    );
-    $self->process_in_file(
-        in      => $tmpl, 
-        out     => $tmp{'config'}{'apache'},
-        options => \%opt,
-    );
-
-    $self->fork_exec($info{'executable'}, '-f', $tmp{'config'}{'apache'});
-    my $pid = do {
-        my $tries = 10;
-        while ( !-e $opt{'pid_file'} ) {
-            $tries--;
-            last unless $tries;
-            sleep 1;
-        }
-        Test::More::BAIL_OUT("Couldn't start apache server, no pid file")
-            unless -e $opt{'pid_file'};
-        open( my $pid_fh, '<', $opt{'pid_file'} )
-            or Test::More::BAIL_OUT("Couldn't open pid file: $!");
-        my $pid = <$pid_fh>;
-        chomp $pid;
-        $pid;
-    };
-
-    Test::More::ok($pid, "Started apache server #$pid");
-
+    require RT::Test::Apache;
+    my $pid = RT::Test::Apache->start_server($variant || 'mod_perl', $port, \%tmp);
     push @SERVERS, $pid;
 
-    return (RT->Config->Get('WebURL'), RT::Test::Web->new);
-}
-
-sub apache_server_info {
-    my $self = shift;
-    my %res = @_;
-
-    my $bin = $res{'executable'} = $ENV{'RT_TEST_APACHE'}
-        || $self->find_apache_server
-        || Test::More::BAIL_OUT("Couldn't find apache server, use RT_TEST_APACHE");
-
-    Test::More::diag("Using '$bin' apache executable for testing")
-        if $ENV{'TEST_VERBOSE'};
-
-    my $info = `$bin -V`;
-    ($res{'version'}) = ($info =~ m{Server\s+version:\s+Apache/(\d+\.\d+)\.});
-    Test::More::BAIL_OUT(
-        "Couldn't figure out version of the server"
-    ) unless $res{'version'};
-
-    my %opts = ($info =~ m/^\s*-D\s+([A-Z_]+?)(?:="(.*)")$/mg);
-    %res = (%res, %opts);
-
-    $res{'modules'} = [
-        map {s/^\s+//; s/\s+$//; $_}
-        grep $_ !~ /Compiled in modules/i,
-        split /\r*\n/, `$bin -l`
-    ];
-
-    return %res;
-}
-
-sub apache_mod_perl_server_options {
-    my $self = shift;
-    my %info = %{ shift() };
-    my $current = shift;
-
-    my %required_modules = (
-        '2.2' => [qw(authz_host log_config env alias perl)],
-    );
-    my @mlist = @{ $required_modules{ $info{'version'} } };
-
-    $current->{'load_modules'} = '';
-    foreach my $mod ( @mlist ) {
-        next if grep $_ =~ /^(mod_|)$mod\.c$/, @{ $info{'modules'} };
-
-        $current->{'load_modules'} .=
-            "LoadModule ${mod}_module modules/mod_${mod}.so\n";
-    }
-    return;
-}
-
-sub apache_fastcgi_server_options {
-    my $self = shift;
-    my %info = %{ shift() };
-    my $current = shift;
-
-    my %required_modules = (
-        '2.2' => [qw(authz_host log_config env alias mime fastcgi)],
-    );
-    my @mlist = @{ $required_modules{ $info{'version'} } };
-
-    $current->{'load_modules'} = '';
-    foreach my $mod ( @mlist ) {
-        next if grep $_ =~ /^(mod_|)$mod\.c$/, @{ $info{'modules'} };
-
-        $current->{'load_modules'} .=
-            "LoadModule ${mod}_module modules/mod_${mod}.so\n";
-    }
-    return;
-}
-
-sub find_apache_server {
-    my $self = shift;
-    return $_ foreach grep defined,
-        map $self->find_executable($_),
-        qw(httpd apache apache2 apache1);
-    return undef;
+    my $url = RT->Config->Get('WebURL');
+    $url =~ s!/$!!;
+    return ($url, RT::Test::Web->new);
 }
 
 sub stop_server {
     my $self = shift;
+    my $in_end = shift;
+    return unless @SERVERS;
 
     my $sig = 'TERM';
-    $sig = 'INT' if !$ENV{'RT_TEST_WEB_HANDLER'}
-                    || $ENV{'RT_TEST_WEB_HANDLER'} =~/^standalone(?:\+|\z)/;
+    $sig = 'INT' if $ENV{'RT_TEST_WEB_HANDLER'} eq "plack";
     kill $sig, @SERVERS;
     foreach my $pid (@SERVERS) {
-        waitpid $pid, 0;
+        if ($ENV{RT_TEST_WEB_HANDLER} =~ /^apache/) {
+            sleep 1 while kill 0, $pid;
+        } else {
+            waitpid $pid, 0;
+        }
     }
+
+    @SERVERS = ();
+}
+
+sub temp_directory {
+    return $tmp{'directory'};
 }
 
 sub file_content {
@@ -1255,44 +1401,9 @@ sub find_executable {
     return undef;
 }
 
-sub fork_exec {
-    my $self = shift;
-
-    my $pid = fork;
-    unless ( defined $pid ) {
-        die "cannot fork: $!";
-    } elsif ( !$pid ) {
-        exec @_;
-        die "can't exec `". join(' ', @_) ."` program: $!";
-    } else {
-        return $pid;
-    }
-}
-
-sub process_in_file {
-    my $self = shift;
-    my %args = ( in => undef, options => undef, @_ );
-
-    my $text = $self->file_content( $args{'in'} );
-    while ( my ($opt) = ($text =~ /\%\%(.+?)\%\%/) ) {
-        my $value = $args{'options'}{ lc $opt };
-        die "no value for $opt" unless defined $value;
-
-        $text =~ s/\%\%\Q$opt\E\%\%/$value/g;
-    }
-
-    my ($out_fh, $out_conf);
-    unless ( $args{'out'} ) {
-        ($out_fh, $out_conf) = tempfile();
-    } else {
-        $out_conf = $args{'out'};
-        open( $out_fh, '>', $out_conf )
-            or die "couldn't open '$out_conf': $!";
-    }
-    print $out_fh $text;
-    seek $out_fh, 0, 0;
-
-    return ($out_fh, $out_conf);
+sub diag {
+    return unless $ENV{RT_TEST_VERBOSE} || $ENV{TEST_VERBOSE};
+    goto \&Test::More::diag;
 }
 
 sub parse_mail {
@@ -1303,6 +1414,14 @@ sub parse_mail {
     return $parser->Entity;
 }
 
+sub works {
+    Test::More::ok($_[0], $_[1] || 'This works');
+}
+
+sub fails {
+    Test::More::ok(!$_[0], $_[1] || 'This should fail');
+}
+
 END {
     my $Test = RT::Test->builder;
     return if $Test->{Original_Pid} != $$;
@@ -1312,10 +1431,10 @@ END {
     # so calls below may call system or kill that clobbers $?
     local $?;
 
-    RT::Test->stop_server;
+    RT::Test->stop_server(1);
 
     # not success
-    if ( !$Test->summary || grep !$_, $Test->summary ) {
+    if ( !$Test->is_passing ) {
         $tmp{'directory'}->unlink_on_destroy(0);
 
         Test::More::diag(
@@ -1325,15 +1444,16 @@ END {
     }
 
     if ( $ENV{RT_TEST_PARALLEL} && $created_new_db ) {
-
-        # Pg doesn't like if you issue a DROP DATABASE while still connected
-        my $dbh = $RT::Handle->dbh;
-        $dbh->disconnect if $dbh;
-
-        $dbh = _get_dbh( RT::Handle->SystemDSN, $ENV{RT_DBA_USER}, $ENV{RT_DBA_PASSWORD} );
-        RT::Handle->DropDatabase( $dbh, Force => 1 );
-        $dbh->disconnect;
+        __drop_database();
     }
 }
 
+{ 
+    # ease the used only once warning
+    no warnings;
+    no strict 'refs';
+    %{'RT::I18N::en_us::Lexicon'};
+    %{'Win32::Locale::Lexicon'};
+}
+
 1;
